var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
import { h } from "vdtree";
import { createFormConfig, getFormHtmlAttrs, AbstractLabel, AbstractInput } from "boost-web-forms";
import { isEmpty, toArray } from "boost-web-core";
function getColumnClass(columns, colSpan, prefix = "col-", maxCols = 12) {
  return columns > 1 ? `${prefix}${maxCols * colSpan / columns}` : "";
}
function BootstrapForm({ forObject, formConfig, validationResult, fieldSet, layoutOptions, htmlAttrs = {} }) {
  var _a, _b, _c;
  let _formConfig = formConfig != null ? formConfig : createFormConfig(forObject);
  let result = h(_formConfig.excludeSubmitButton ? "div" : "form", __spreadValues(__spreadValues({}, getFormHtmlAttrs(_formConfig)), htmlAttrs));
  (_b = (_a = result.props).class) != null ? _b : _a.class = "";
  if (layoutOptions.columns > 1)
    result.props.class += " row";
  for (const fieldId in _formConfig.fieldsConfig) {
    if (!_formConfig.fieldsConfig.hasOwnProperty(fieldId))
      continue;
    let field = _formConfig.fieldsConfig[fieldId];
    let fieldValidationResult = (_c = validationResult == null ? void 0 : validationResult.fields[fieldId]) != null ? _c : { hasError: false };
    let fSet = fieldSet({ field, value: forObject[field.id], fieldValidationResult, layoutOptions });
    result.children.push(fSet);
  }
  return result;
}
const classTable = {
  checkbox: "form-check-input",
  radio: "form-check-input",
  file: "form-control-file",
  files: "form-control-file",
  range: "form-control-range",
  submit: "btn btn-primary",
  select: "custom-select"
};
function Bootstrap4Form(layoutOptions) {
  let _layoutOptions = __spreadValues({
    columns: 1,
    horizontalLabels: false,
    isInline: false
  }, layoutOptions);
  let layout = function(props) {
    let htmlAttrs = {};
    if (_layoutOptions.isInline)
      htmlAttrs.class = "form-inline";
    return BootstrapForm(__spreadProps(__spreadValues({}, props), {
      fieldSet: Bootstrap4FieldSet,
      layoutOptions: _layoutOptions,
      htmlAttrs
    }));
  };
  return { layout };
}
function Bootstrap4FieldSet({ field, value, fieldValidationResult, layoutOptions }) {
  let fieldSet = h("div", { class: `${field.type === "checkbox" ? `form-check ${layoutOptions.isInline ? "form-check-inline" : ""}` : ""}` });
  const isCheckOrRadio = field.type === "checkbox" || field.type === "radio";
  let inputClass = classTable[field.type] || "form-control";
  if (fieldValidationResult.hasError)
    inputClass += " is-invalid";
  if (field.scale != 1 && !isCheckOrRadio) {
    if (field.type == "submit")
      inputClass += ` btn-${field.scale > 1 ? "lg" : "sm"}`;
    else
      inputClass += ` form-control-${field.scale > 1 ? "lg" : "sm"}`;
  }
  if (layoutOptions.isInline && isEmpty(field.placeholder))
    field.placeholder = field.label;
  let label = AbstractLabel({
    field,
    validationResult: fieldValidationResult,
    htmlAttrs: { class: `${isCheckOrRadio ? "form-check-label" : ""} ${layoutOptions.isInline ? "sr-only" : ""}` }
  });
  let input = AbstractInput({
    field,
    value,
    validationResult: fieldValidationResult,
    htmlAttrs: { class: inputClass }
  });
  if (field.type === "radio") {
    input = Object.keys(field.choices).map((k, i) => h("div", { class: `form-check ${layoutOptions.isInline ? "form-check-inline" : ""}` }, h("label", {}, toArray(input)[i], " ", field.choices[k])));
  }
  let colClass = getColumnClass(layoutOptions.columns, field.type == "submit" ? layoutOptions.columns : field.colSpan);
  if (field.type != "checkbox" || field.readonly)
    fieldSet.children.push(...toArray(label), " ", ...toArray(input));
  else
    fieldSet.children.push(...toArray(input), " ", ...toArray(label));
  if (fieldValidationResult && fieldValidationResult.hasError)
    fieldSet.children.push(h("div", { class: "invalid-feedback" }, fieldValidationResult.message));
  if (!isEmpty(colClass))
    fieldSet = h("div", { class: `mb-2 ${colClass}` }, fieldSet);
  else
    fieldSet.props.class += " mb-2";
  if (!isEmpty(field.helpText))
    fieldSet.children.push(h("small", { class: "form-text text-muted" }, field.helpText));
  if (layoutOptions.isInline)
    fieldSet.props.class += " mb-2 mr-sm-2";
  return fieldSet;
}
export { Bootstrap4Form };
